/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.exports.process;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.jfree.chart.annotations.XYTextAnnotation;
import org.jfree.data.xy.XYSeries;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.access.RiverAccess;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.model.WQKms;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.exports.DischargeCurveGenerator;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.exports.XYChartGenerator;
import org.dive4elements.river.jfree.CollisionFreeXYTextAnnotation;
import org.dive4elements.river.jfree.RiverAnnotation;
import org.dive4elements.river.jfree.StickyAxisAnnotation;
import org.dive4elements.river.jfree.StyledXYSeries;
import org.dive4elements.river.model.Gauge;
import org.dive4elements.river.model.River;
import org.dive4elements.river.themes.ThemeDocument;


/** Helper for data handling in discharge diagrams. */
public class DischargeProcessor
extends DefaultProcessor implements FacetTypes {

    private final static Logger logger =
            Logger.getLogger(DischargeProcessor.class);

    /** Station for which the diagram is shown. */
    private double km;

    /** Tolerance for comparison of kilometers. */
    public static final double KM_EPSILON = 0.001d;


    /** This processor needs to be constructed with a given km. */
    public DischargeProcessor() {
        km = Double.NaN;
    }


    public DischargeProcessor(double km) {
        this.km = km;
    }


    /** Process data, add it to plot. */
    @Override
    public void doOut(
            DiagramGenerator generator,
            ArtifactAndFacet bundle,
            ThemeDocument theme,
            boolean visible
    ) {
        CallContext context = generator.getCallContext();
        Object data = bundle.getData(context);
        if (data instanceof WQKms) {
            doDischargeOut(
                generator,
                bundle,
                bundle.getFacetName(),
                theme,
                visible);
        }
        else {
            logger.error("Can't process "
                + data.getClass().getName() + " objects of facet "
                + bundle.getFacetName());
        }
    }

    /** True if this processor knows how to deal with facetType. */
    @Override
    public boolean canHandle(String facetType) {
        return DISCHARGE_CURVE.equals(facetType)
            || GAUGE_DISCHARGE_CURVE.equals(facetType);
    }


    /** The station of the current calculation/view. */
    protected double getKm() {
        return km;
    }

    /**
     * Add series with discharge curve to diagram.
     */
    protected void doDischargeOut(
        DiagramGenerator generator,
        ArtifactAndFacet bundle,
        String        name,
        ThemeDocument theme,
        boolean       visible)
    {
        logger.debug("DischargeCurveGenerator.doDischargeOut");
        CallContext context = generator.getCallContext();
        WQKms data = (WQKms)bundle.getData(context);
        XYSeries series = new StyledXYSeries(name, theme);
        double[][] wqData = new double[2][data.size()];
        for (int i = 0; i < data.size(); i++) {
            wqData[0][i] = data.getQ(i);
            wqData[1][i] = data.getW(i);
        }
        StyledSeriesBuilder.addPoints(series, wqData, false);

        String gaugeName = data.getName();
        RiverAccess access = new RiverAccess((D4EArtifact)bundle.getArtifact());
        River river = access.getRiver();

        if (river == null) {
            logger.debug("no river found");
            return;
        }

        Gauge gauge = river.determineGaugeByName(gaugeName);

        if (gauge == null) {
            logger.debug("no gauge found");
            return;
        }

        generator.addAxisSeries(series, axisName, visible);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
